<?php

// diese klasse dient als schnittstelle zu einem TO Server mit TOST 1.4


class toserver {
	
	var $strIP;
	var $intPort;
	var $strPass;

	var $intMaxAttempts = 3;
	var $intStreamTimeout = 3;
	
	var $strError = false;

	function toserver($strIP, $intPort, $strPass) {
		$this->strIP = $strIP;
		$this->intPort = $intPort;
		$this->strPass = $strPass;
		if ($intMaxAttempts) $this->intMaxAttempts = $intMaxAttempts;
		if ($intStreamTimeout) $this->intStreamTimeout = $intStreamTimeout;
	}

	// sends a "say" message to the server
	
	function say($strMessage) {
		return $this->do_safe_query("say {$strMessage}");
	}

	// sets a variable on the gameserver and returns true on success, fals on failure
	// example:   set_server_var("Engine.Gameinfo", "GamePassword", "mygamepass")

	function set_server_var($strClass, $strVar, $strValue) {

		// remove slashes, because the TOST protocol filters them out anyways
		$strClass = str_replace("\\", "", $strClass);
		$strVar = str_replace("\\", "", $strVar);
		$strValue = str_replace("\\", "", $strValue);

		// turn , into . because thats what the TOST protocol does, too
		$strClass = str_replace(",", ".", $strClass);
		$strVar = str_replace(",", ".", $strVar);
		$strValue = str_replace(",", ".", $strValue);

		if (empty($strClass)) {
			die ("fatal internal error in query syntax: no classname specified");
		}
		if (empty($strVar)) {
			die ("fatal internal error in query syntax: no variable name specified");
		}
	
		$strSetCommand = "set {$strClass} {$strVar} {$strValue}";
		$arrSetReply = $this->do_safe_query($strSetCommand);	// set the variable

		if (!is_array($arrSetReply)) return false;

		$strGetReply = $this->get_server_var($strClass, $strVar);
		
		if ($strGetReply !== $strValue) {
			$this->strError = "udp://{$this->strIP}:{$this->intPort} says: ".$strGetReply;
			return false;
		}

		return true;

	}


	function get_server_var($strClass, $strVar) {

		$strClass = str_replace("\\", "", $strClass);
		$strVar = str_replace("\\", "", $strVar);

		if (empty($strClass)) {
			die ("fatal internal error in query syntax: no classname specified");
		}
		if (empty($strVar)) {
			die ("fatal internal error in query syntax: no variable name specified");
		}
	
		$strGetCommand = "get {$strClass} {$strVar}";

		$arrGetReply = $this->do_safe_query($strGetCommand);	
		return $arrGetReply[$strGetCommand];

	}

	// calls do_query() until it either returns a correctly formulated answer, or $intMaxAttempts is reached

	function do_safe_query($strCommand) {
		for ($intAttempts = 0; true ; $intAttempts ++) {
			$arrReply = $this->do_query($strCommand);
			if (is_array($arrReply)) return $arrReply;
			if ( (!is_array($arrReply)) AND ($intAttempts >= $this->intMaxAttempts) ) return false;
		}
	}

	// does a console query
	function do_query($strCommand) {
		$strQuery = '\\'.$this->strPass.'\\console\\'.$strCommand.'\\';
		return $this->do_server_query($strQuery);
	}

	// queries the server and returns an array containing the answer, or false on error
	function do_server_query($strQuery) {
		$fp = @fsockopen("udp://{$this->strIP}", $this->intPort, $intError, $strError, 5);
		if (!is_resource($fp)) {
			$this->strError = "invalid server address: udp://{$this->strIP}:{$this->intPort}";
			return false;
		}
		stream_set_timeout($fp, $this->intStreamTimeout);
		fwrite($fp,$strQuery);
	
		while (true) {
			$chrIn = fread($fp, 1);
			if ($chrIn == "") break;
			$strIn .= $chrIn;
			if (Substr($strIn, strlen($strIn)-7, 7) == '\\final\\') break;
		}
		fclose($fp);

		// $strIn is what the server responded
	
		// cut off "\final\" and first slash
		$strIn = Substr($strIn,1, strlen($strIn)-8);
	
		$arrIn = explode("\\", $strIn);
	
		if (count($arrIn) < 2) {
			$this->strError = "Server did not respond (either server is offline or query password is wrong) udp://{$this->strIP}:{$this->intPort}\n";
			return false;
		}
	
		$intCount = 0;
		while (is_string($arrIn[$intCount])) {
			$arrRules[$arrIn[$intCount]] = $arrIn[$intCount+1];
			$intCount = $intCount +2;
		}
		return $arrRules;
	}

        function set_multiple_server_vars($arrVars) {
                if (!is_array($arrVars)) return false;

                foreach ($arrVars as $arrPacket) {
                        $bSuccess = $this->set_server_var($arrPacket['class'], $arrPacket['variable'], $arrPacket['value']);
                        if (!$bSuccess) return false;
                }
                return true;
        }

}
?>
